@@ -9,6 +9,7 @@ from andes.io.txt import dump_data
 from andes.utils.misc import elapsed
 from andes.routines.base import BaseRoutine
 from andes.shared import np, matrix, spmatrix, plt, mpl
+from andes.plot import set_latex
 
 logger = logging.getLogger(__name__)
 __cli__ = 'eig'
@@ -30,6 +31,8 @@ class EIG(BaseRoutine):
         self.eigs = None
         self.mu = None
         self.part_fact = None
+        self.singular_idx = np.array([], dtype=int)
+        self.x_name = []
 
     def calc_state_matrix(self):
         r"""
@@ -60,10 +63,18 @@ class EIG(BaseRoutine):
         gyx = matrix(system.dae.gx)
         self.solver.linsolve(system.dae.gy, gyx)
 
-        iTf = spdiag((1 / system.dae.Tf).tolist())
+        Tfnz = system.dae.Tf + np.ones_like(system.dae.Tf) * np.equal(system.dae.Tf, 0.0)
+        iTf = spdiag((1 / Tfnz).tolist())
         self.As = matrix(iTf * (system.dae.fx - system.dae.fy * gyx))
         return self.As
 
+    def remove_singular_rc(self):
+        """
+        Remove rows and cols associated with zero time constant.
+        """
+        self.As = np.delete(self.As, self.singular_idx, axis=0)
+        self.As = np.delete(self.As, self.singular_idx, axis=1)
+
     def calc_eigvals(self):
         """
         Solve eigenvalues of the state matrix ``self.As``
@@ -92,7 +103,7 @@ class EIG(BaseRoutine):
         idx = range(n)
 
         mu_complex = np.array([0] * n, dtype=complex)
-        W = matrix(spmatrix(1.0, idx, idx, (n, n), N.typecode))
+        W = matrix(spmatrix(1.0, idx, idx, As.shape, N.typecode))
         gesv(N, W)
 
         partfact = mul(abs(W.T), abs(N))
@@ -104,7 +115,7 @@ class EIG(BaseRoutine):
         for item in idx:
             mu_real = float(mu[item].real)
             mu_imag = float(mu[item].imag)
-            mu_complex[item] = complex(round(mu_real, 4), round(mu_imag, 4))
+            mu_complex[item] = complex(round(mu_real, 5), round(mu_imag, 5))
             partfact[item, :] /= WN[item]
 
         # participation factor
@@ -124,47 +135,54 @@ class EIG(BaseRoutine):
         logger.info(out_str)
 
     def run(self, **kwargs):
-        ret = False
+        succeed = False
         system = self.system
+        self.singular_idx = np.array([], dtype=int)
 
         if system.PFlow.converged is False:
             logger.warning('Power flow not solved. Eig analysis will not continue.')
-            return ret
+            return succeed
         else:
             if system.TDS.initialized is False:
                 system.TDS.init()
+                system.TDS._itm_step()
 
         if system.dae.n == 0:
-            logger.warning('No dynamic model. Eig analysis will not continue.')
-            return ret
+            logger.error('No dynamic model. Eig analysis will not continue.')
 
-        if sum(system.dae.Tf != 0) != len(system.dae.Tf):
-            logger.error("System contains zero time constant(s). "
-                         "Eigenvalue analysis cannot continue.")
-            return ret
+        else:
+            if sum(system.dae.Tf != 0) != len(system.dae.Tf):
+                self.singular_idx = np.argwhere(np.equal(system.dae.Tf, 0.0)).ravel().astype(int)
+                logger.info(f"System contains {len(self.singular_idx)} zero time constants. ")
+                logger.debug([system.dae.x_name[i] for i in self.singular_idx])
 
-        self.summary()
-        t1, s = elapsed()
+            self.x_name = np.array(system.dae.x_name)
+            self.x_name = np.delete(self.x_name, self.singular_idx)
 
-        self.calc_state_matrix()
-        self.calc_part_factor()
+            self.summary()
+            t1, s = elapsed()
 
-        if not self.system.files.no_output:
-            self.report()
-            if system.options.get('state_matrix') is True:
-                self.export_state_matrix()
+            self.calc_state_matrix()
+            self.remove_singular_rc()
+            self.calc_part_factor()
 
-        if self.config.plot:
-            self.plot()
-        ret = True
+            if not self.system.files.no_output:
+                self.report()
+                if system.options.get('state_matrix') is True:
+                    self.export_state_matrix()
 
-        _, s = elapsed(t1)
-        logger.info('Eigenvalue analysis finished in {:s}.'.format(s))
+            if self.config.plot:
+                self.plot()
+            _, s = elapsed(t1)
+            logger.info('Eigenvalue analysis finished in {:s}.'.format(s))
 
-        return ret
+            succeed = True
+
+        system.exit_code = 0 if succeed else 1
+        return succeed
 
     def plot(self, mu=None, fig=None, ax=None, left=-6, right=0.5, ymin=-8, ymax=8, damping=0.05,
-             linewidth=0.5, dpi=150):
+             line_width=0.5, dpi=150, show=True, latex=True):
         mpl.rc('font', family='Times New Roman', size=12)
 
         if mu is None:
@@ -194,9 +212,11 @@ class EIG(BaseRoutine):
             logger.info(
                 'System is small-signal stable in the initial neighbourhood.')
 
-        mpl.rc('text', usetex=True)
+        set_latex(latex)
+
         if fig is None or ax is None:
             fig, ax = plt.subplots(dpi=dpi)
+
         ax.scatter(n_mu_real, n_mu_imag, marker='x', s=40, linewidth=0.5, color='black')
         ax.scatter(z_mu_real, z_mu_imag, marker='o', s=40, linewidth=0.5, facecolors='none', edgecolors='black')
         ax.scatter(p_mu_real, p_mu_imag, marker='x', s=40, linewidth=0.5, color='black')
@@ -208,14 +228,15 @@ class EIG(BaseRoutine):
         yneg = xin / damping
         ypos = - xin / damping
 
-        ax.plot(xin, yneg, color='grey', linewidth=linewidth, linestyle='--')
-        ax.plot(xin, ypos, color='grey', linewidth=linewidth, linestyle='--')
+        ax.plot(xin, yneg, color='grey', linewidth=line_width, linestyle='--')
+        ax.plot(xin, ypos, color='grey', linewidth=line_width, linestyle='--')
         ax.set_xlabel('Real')
         ax.set_ylabel('Imaginary')
         ax.set_xlim(left=left, right=right)
         ax.set_ylim(ymin, ymax)
 
-        # plt.show()
+        if show is True:
+            plt.show()
         return fig, ax
 
     def export_state_matrix(self):
@@ -239,7 +260,7 @@ class EIG(BaseRoutine):
         logger.info(f'State matrix saved to "{system.files.mat}".')
         return True
 
-    def report(self):
+    def report(self, x_name=None):
         """
         Save eigenvalue analysis reports
 
@@ -247,9 +268,13 @@ class EIG(BaseRoutine):
         -------
         None
         """
+        from andes.variables.report import report_info
+
         system = self.system
         mu = self.mu
         part_fact = self.part_fact
+        if x_name is None:
+            x_name = self.x_name
 
         text = []
         header = []
@@ -292,7 +317,7 @@ class EIG(BaseRoutine):
         for prow in range(neig):
             temp_row = part_fact[prow, :]
             name_idx = list(temp_row).index(max(temp_row))
-            var_assoc.append(system.dae.x_name[name_idx])
+            var_assoc.append(x_name[name_idx])
 
         pf = []
         for prow in range(neig):
@@ -301,7 +326,13 @@ class EIG(BaseRoutine):
                 temp_row.append(round(part_fact[prow, pcol], 5))
             pf.append(temp_row)
 
+        # opening info section
+        text.append(report_info(self.system))
+        header.append(None)
+        rowname.append(None)
+        data.append(None)
         text.append('')
+
         header.append([''])
         rowname.append(['EIGENVALUE ANALYSIS REPORT'])
         data.append('')
@@ -313,14 +344,20 @@ class EIG(BaseRoutine):
 
         text.append('EIGENVALUE DATA\n')
         header.append([
-            'Most Associated', 'Real', 'Imag', 'Damped Freq.', 'Frequency',
-            'Damping [%]'
-        ])
+            'Most Associated',
+            'Real',
+            'Imag.',
+            'Damped Freq.',
+            'Frequency',
+            'Damping [%]'])
         rowname.append(numeral)
         data.append(
             [var_assoc,
              list(mu_real),
-             list(mu_imag), ufreq, freq, damping])
+             list(mu_imag),
+             ufreq,
+             freq,
+             damping])
 
         n_cols = 7  # columns per block
         n_block = int(ceil(neig / n_cols))
@@ -332,7 +369,7 @@ class EIG(BaseRoutine):
                 text.append('PARTICIPATION FACTORS [{}/{}]\n'.format(
                     idx + 1, n_block))
                 header.append(numeral[start:end])
-                rowname.append(system.dae.x_name)
+                rowname.append(x_name)
                 data.append(pf[start:end])
 
         dump_data(text, header, rowname, data, system.files.eig)
