@@ -1,6 +1,6 @@
 import logging
 from collections import OrderedDict
-from andes.core.triplet import JacTriplet
+from andes.core import JacTriplet
 from andes.shared import pd, np, spmatrix, jac_names
 
 logger = logging.getLogger(__name__)
@@ -14,20 +14,17 @@ class DAETimeSeries(object):
         self.dae = dae
 
         # internal dict storage
-        self._data = OrderedDict()
+        self._xy = OrderedDict()
         self._z = OrderedDict()
 
         self.t = np.array([])
-        self.xy = np.array([])
-        self.z = np.array([])
+        self.xy = np.array([]).reshape((-1, 1))
+        self.z = np.array([]).reshape((-1, 1))
 
         # data frame members
         self.df = None
         self.df_z = None
 
-        # flags
-        self._need_unpack = True
-
     @property
     def x(self):
         return self.xy[:, 0:self.dae.n+1]
@@ -42,22 +39,42 @@ class DAETimeSeries(object):
         """
         Return the values of [t, x, y, z] in an array.
         """
-        self.unpack()
+        if len(self._xy) != len(self.t):
+            self.unpack()
+
         if len(self._z):
             return np.hstack((self.t.reshape((-1, 1)), self.xy, self.z))
         else:
             return np.hstack((self.t.reshape((-1, 1)), self.xy))
 
-    def unpack(self):
+    def unpack(self, df=False):
         """
-        Unpack data and make DataFrames and arrays.
-        """
-        self.df = pd.DataFrame.from_dict(self._data, orient='index', columns=self.dae.xy_name)
-        self.t = self.df.index.to_numpy()
-        self.xy = self.df.to_numpy()
+        Unpack stored data in `_xy` and `_z` into arrays `t`, `xy`, and `z`.
 
-        self.df_z = pd.DataFrame.from_dict(self._z, orient='index', columns=self.dae.z_name)
-        self.z = self.df_z.to_numpy()
+        Parameters
+        ----------
+        df : bool
+            True to construct DataFrames `self.df` and `self.df_z` (time-consuming).
+        """
+        if df is True:
+            self.df = pd.DataFrame.from_dict(self._xy, orient='index', columns=self.dae.xy_name)
+            self.t = self.df.index.to_numpy()
+            self.xy = self.df.to_numpy()
+
+            self.df_z = pd.DataFrame.from_dict(self._z, orient='index', columns=self.dae.z_name)
+            self.z = self.df_z.to_numpy()
+        else:
+            n_steps = len(self._xy)
+            self.t = np.array(list(self._xy.keys()))
+            self.xy = np.zeros((n_steps, self.dae.m + self.dae.n))
+            self.z = np.zeros((n_steps, self.dae.o))
+
+            # TODO: write test.
+            for idx, xy in enumerate(self._xy.values()):
+                self.xy[idx, :] = xy
+
+            for idx, z in enumerate(self._z.values()):
+                self.z[idx, :] = z
 
     def store_txyz(self, t, xy, z=None):
         """
@@ -72,7 +89,7 @@ class DAETimeSeries(object):
         z : array-like or None
             discrete flags data
         """
-        self._data[t] = xy
+        self._xy[t] = xy
         if z is not None:
             self._z[t] = z
 
@@ -131,7 +148,7 @@ class DAE(object):
     """
     def __init__(self, system):
         self.system = system
-        self.t = 0
+        self.t = np.array(0)
         self.ts = DAETimeSeries(self)
 
         self.m, self.n, self.o = 0, 0, 0
@@ -153,7 +170,7 @@ class DAE(object):
     def clear_ts(self):
         self.ts = DAETimeSeries(self)
 
-    def clear_array(self):
+    def clear_arrays(self):
         """
         Reset equation and variable arrays to empty.
         """
@@ -164,23 +181,21 @@ class DAE(object):
     def clear_fg(self):
         """Resets equation arrays to empty
         """
-        self.f = np.zeros(self.n)
-        self.g = np.zeros(self.m)
+        self.f[:] = 0
+        self.g[:] = 0
 
     def clear_xy(self):
         """
         Reset variable arrays to empty.
-
-        TODO: use in-place assignment instead of reassignment, where an extra resizing is needed.
         """
-        self.x = np.zeros(self.n)
-        self.y = np.zeros(self.m)
+        self.x[:] = 0
+        self.y[:] = 0
 
     def clear_z(self):
         """
         Reset status arrays to empty
         """
-        self.z = np.zeros(self.n)
+        self.z[:] = 0
 
     def clear_ijv(self):
         """
@@ -196,16 +211,17 @@ class DAE(object):
             self.build_pattern(jname)
 
     def reset(self):
+        """
+        Reset array sizes to zero and clear all arrays.
+        """
+        self.t = 0
         self.m = 0
         self.n = 0
-        self.clear_fg()
-        self.clear_xy()
-        self.clear_z()
+        self.o = 0
+        self.resize_arrays()
         self.clear_ijv()
         self.clear_ts()
 
-        self.Tf = np.ones(self.n)
-
     def get_size(self, name):
         """
         Get the size of an array or sparse matrix based on name.
@@ -263,10 +279,14 @@ class DAE(object):
         name : name
             jac name
         """
-        self.__dict__[name] = spmatrix(self.triplets.vjac[name],
-                                       self.triplets.ijac[name],
-                                       self.triplets.jjac[name],
-                                       self.get_size(name), 'd')
+        try:
+            self.__dict__[name] = spmatrix(self.triplets.vjac[name],
+                                           self.triplets.ijac[name],
+                                           self.triplets.jjac[name],
+                                           self.get_size(name), 'd')
+        except TypeError as e:
+            logger.error("Your new model might have accessed an Algeb using ExtState, or vice versa.")
+            raise e
 
     def _compare_pattern(self, name):
         """
@@ -288,21 +308,30 @@ class DAE(object):
             if m_after.I[i] != m_before.I[i] or m_after.J[i] != m_before.J[i]:
                 raise KeyError
 
-    def resize_array(self):
+    def resize_arrays(self):
         """
-        Resize arrays to the new `m` and `n`
-
-        Returns
-        -------
+        Resize arrays to the new sizes `m` and `n`, and `o`.
 
+        If ``m > len(self.y)`` or ``n > len(self.x``, arrays will be extended.
+        Otherwise, new empty arrays will be sliced, starting from 0 to the given size.
         """
-        self.x = np.append(self.x, np.zeros(self.n - len(self.x)))
-        self.y = np.append(self.y, np.zeros(self.m - len(self.y)))
-        self.z = np.append(self.z, np.zeros(self.o - len(self.z)))
+        self.x = self._extend_or_slice(self.x, self.n)
+        self.y = self._extend_or_slice(self.y, self.m)
+        self.z = self._extend_or_slice(self.z, self.o)
 
-        self.f = np.append(self.f, np.zeros(self.n - len(self.f)))
-        self.g = np.append(self.g, np.zeros(self.m - len(self.g)))
-        self.Tf = np.append(self.Tf, np.ones(self.n - len(self.Tf)))
+        self.f = self._extend_or_slice(self.f, self.n)
+        self.g = self._extend_or_slice(self.g, self.m)
+        self.Tf = self._extend_or_slice(self.Tf, self.n, fill_func=np.ones)
+
+    def _extend_or_slice(self, array, new_size, fill_func=np.zeros):
+        """
+        Helper function for ``self.resize_arrays`` to grow or shrink arrays.
+        """
+        if new_size > len(array):
+            array = np.append(array, fill_func(new_size - len(array)))
+        else:
+            array = array[0:new_size]
+        return array
 
     @property
     def xy(self):
@@ -388,4 +417,15 @@ class DAE(object):
         return True
 
     def write_npy(self, npy_path):
-        np.save(npy_path, self.ts.txyz)
+        """
+        Write TDS data into NumPy output files.
+
+        TODO: Compress when the matrix is larger than 25 MB.
+        """
+        txyz_data = self.ts.txyz
+
+        # if txyz_data.size > 3125000:  # 25 megabytes of flaot64
+        #     np.savez_compressed(npy_path, txyz_data)
+        # else:
+
+        np.save(npy_path, txyz_data)
